-----------------------------------------------------------------------
-- Locale
-----------------------------------------------------------------------
local L = AceLibrary("AceLocale-2.2"):new("Violation: Banzai")

local Banzai = AceLibrary("Banzai-1.1")
local Roster = AceLibrary("Roster-2.1")

-----------------------------------------------------------------------
-- Locals
-----------------------------------------------------------------------

local moduleName = "Banzai"
local gainedAggroTime

local PERIODICUPDATE_ID = "Violation-"..moduleName.."-PeriodicUpdate"

-----------------------------------------------------------------------
-- Module declaration
-----------------------------------------------------------------------

local mod = Violation:NewModule(moduleName)
mod.revision = tonumber(("$Revision: 38177 $"):sub(12, -3)) or 1
mod.version = 4
mod.violationCategory = "other"

-----------------------------------------------------------------------
-- Module initialization
-----------------------------------------------------------------------

function mod:OnModuleEnable()
	AceLibrary("Banzai-1.1") -- Ensure it is loaded
	self:RegisterEvent("Banzai_UnitGainedAggro")
	self:RegisterEvent("Banzai_UnitLostAggro")	
	self:SetupAggroTable()
end

function mod:OnModuleDisable()
	self:ClearAggroTable()
	self:CancelAllScheduledEvents()
end

function mod:OnReset()
	gainedAggroTime = mod:del(gainedAggroTime)
end

-----------------------------------------------------------------------
-- Module implementation
-----------------------------------------------------------------------

function mod:GetDisplayName() return "Aggro duration" end
function mod:GetShortDisplayName() return "Aggro" end

function mod:IsDirty()
	return self.dirty or (gainedAggroTime and next(gainedAggroTime))
end

function mod:IsBroadcastDirty()
	return self.broadcastdirty or (gainedAggroTime and gainedAggroTime[self.playerName])
end

function mod:ClearAggroTable()
	if gainedAggroTime then
		for name in pairs(gainedAggroTime) do
			self:LostAggro(name)
		end
		gainedAggroTime = mod:del(gainedAggroTime)
	end
end

function mod:SetupAggroTable()
	gainedAggroTime = mod:del(gainedAggroTime) -- we never know
	gainedAggroTime = mod:new()
	for unit in Roster:IterateRoster(true) do
		if Banzai:GetUnitAggroByUnitName(unit.name) then
			self:GainedAggro(unit.name)
		end
	end
end

function mod:UpdateData(name)
	if not gainedAggroTime[name] then return end
	local now = GetTime()
	local delta = now - gainedAggroTime[name]
	gainedAggroTime[name] = now
	self:OnData(name, delta)
end

function mod:LostAggro(name)
	if self:AcceptDataFor(name) and gainedAggroTime and gainedAggroTime[name] then
		self:UpdateData(name)
		gainedAggroTime[name] = nil
	end
end

function mod:GainedAggro(name)
	if self:AcceptDataFor(name) and (not gainedAggroTime or not gainedAggroTime[name]) then
		if not gainedAggroTime then
			gainedAggroTime = self:new()
		end
		gainedAggroTime[name] = GetTime()
		self:UpdateData(name)
	end
	if next(gainedAggroTime) and not self:IsEventScheduled(PERIODICUPDATE_ID) then
		self:ScheduleRepeatingEvent(PERIODICUPDATE_ID, self.PeriodicUpdate, 1, self) 
	end
end

function mod:Banzai_UnitGainedAggro(id) 
	local unit = Roster:GetUnitObjectFromUnit(id)
	self:GainedAggro(unit.name) 
end

function mod:Banzai_UnitLostAggro(id) 
	local unit = Roster:GetUnitObjectFromUnit(id)
	self:LostAggro(unit.name)
end


function mod:PeriodicUpdate()
	for name in pairs(gainedAggroTime) do		
		self:UpdateData(name)
	end
	if not next(gainedAggroTime) and self:IsEventScheduled(PERIODICUPDATE_ID) then
		self:CancelScheduledEvent(PERIODICUPDATE_ID) 
	end
end

local floor = math.floor
function mod.dataFormatter(duration)
	if duration >= 3600 then
		return ("%d:%02d:%02d"):format(floor(duration / 3600), floor(duration/60) % 60, floor(duration) % 60)
	else
		return ("%d:%02d"):format(floor(duration / 60), floor(duration) % 60)
	end
end